from autorml.subject_map import define_subject_column_subject_map, \
    define_object_column_subject_map 
from autorml.predicate_object_map import define_subject_column_po_map, \
    define_object_column_po_map


def define_term_maps(mappings, subject_column, primary_annotations,
                     secondary_annotations, cea, cpa, cta, col_names):

    mappings["mappings"] = {}
    mappings["mappings"]["subjectTM"] = \
        {"sources": ["sem-table"]} if cea else {"sources": ["table"]}
    mappings["mappings"]["subjectTM"] = define_subject_column_subject_map(
                                            mappings["mappings"]["subjectTM"],
                                            subject_column,
                                            secondary_annotations,
                                            cea,
                                            col_names)
    mappings["mappings"]["subjectTM"] = define_subject_column_po_map(
                                            mappings["mappings"]["subjectTM"],
                                            subject_column,
                                            primary_annotations,
                                            secondary_annotations,
                                            cea, cpa, cta,
                                            col_names)
    if isinstance(subject_column, int):
        subject_column = [subject_column]

    for column in range(len(primary_annotations)):
        primary_condition = primary_annotations[column] == "NE"
        if column not in subject_column and primary_condition:
            mappings["mappings"][col_names[column]+"TM"] = \
                {"sources": ["sem-table"]} if cea else {"sources": ["table"]}
            mappings["mappings"][col_names[column]+"TM"] =\
                define_object_column_subject_map(
                    mappings["mappings"][col_names[column]+"TM"],
                    column, primary_annotations, secondary_annotations,
                    cea, col_names[column])
            mappings["mappings"][col_names[column]+"TM"] =\
                define_object_column_po_map(
                    mappings["mappings"][col_names[column]+"TM"],
                    column, cta, col_names[column])

    return mappings



import pandas as pd
import os
from urllib.parse import urlparse


def define_source(mappings, table, cea, primary_annotations):

    df = load_table(table)
    # avoid yatter 'type' error translation
    df.columns = df.columns.str.replace('type', 'typ', regex=True)
    df.columns = df.columns.str.replace('Type', 'typ', regex=True)
    # avoid yatter '()' error translation
    df.columns = df.columns.str.replace(r'\(.*?\)', '', regex=True)
    col_names = df.columns.to_list()

    if cea == []:
        mappings["sources"] = {}

        mappings["sources"]["table"] = {}
        mappings["sources"]["table"]["access"] = table.replace("\\", "\\\\")
        mappings["sources"]["table"]["referenceFormulation"] = "csv"
        mappings["sources"]["table"]["iterator"] = "$"

    else:
        df = generate_sem_table(table, df, cea, primary_annotations)

        mappings["sources"] = {}

        mappings["sources"]["table"] = {}
        mappings["sources"]["table"]["access"] = table.replace("\\", "\\\\")
        mappings["sources"]["table"]["referenceFormulation"] = "csv"
        mappings["sources"]["table"]["iterator"] = "$"

        mappings["sources"]["sem-table"] = {}
        mappings["sources"]["sem-table"]["access"] =\
            table.replace(".csv", "-semantic.csv").replace("\\", "\\\\")
        mappings["sources"]["sem-table"]["referenceFormulation"] = "csv"
        mappings["sources"]["sem-table"]["iterator"] = "$"

    return mappings, col_names


def generate_sem_table(table, df, cea, primary_annotations):

    ne_cols = []
    for i in range(len(primary_annotations)):
        if primary_annotations[i] == "NE":
            ne_cols.append(i)

    for row_idx, col_idx, value in cea:
        df.iat[int(row_idx)-1, int(col_idx)] = value

    # Replace invalid URLs with None
    for idx in ne_cols:
        column = df.columns[idx]
        df[column] =\
            df[column].apply(lambda x: x if pd.isna(x) or is_valid_url(x)
                             else None)

    df.to_csv(os.path.abspath(table.replace(".csv", "-semantic.csv")),
              index=False)

    return


def load_table(file_path):

    """Load CSV input file"""

    # Read the first line
    with open(file_path, 'r', encoding='utf-8') as file:
        first_line = file.readline().strip()

    # Check if the first line is entirely integers
    if all(item.isdigit() for item in first_line.split(',')):
        # Skip the first line if it contains only integers
        df = pd.read_csv(file_path, encoding='utf-8', skiprows=1,
                         quotechar='"', skipinitialspace=True)
    else:
        # Read the file normally
        df = pd.read_csv(file_path, encoding='utf-8', quotechar='"',
                         skipinitialspace=True)

    return df


# Function to validate a URL
def is_valid_url(url):
    try:
        result = urlparse(url)
        return all([result.scheme, result.netloc])
    except ValueError:
        return False


def delete_semantic_table(semantic_table_path):

    semantic_table_path = semantic_table_path
    if os.path.exists(semantic_table_path):
        os.remove(semantic_table_path)

    return



# Definition of the Subject Map for the Subject Column of the table
def define_subject_column_subject_map(subjectTM, subject_column,
                                      secondary_annotations, cea, col_names):

    # If no Subject Column within table, no subject defined which
    # will lead to the creation of blank nodes for subject
    if subject_column == -1 or subject_column is None:
        subjectTM["s"] = [{"value": "Blank", "type": "blank"}]
        return subjectTM

    # Define the reference of the subject column(s) in YARRRML
    if isinstance(subject_column, int):
        subject_column_ref = "$({})".format(col_names[subject_column])
    else:
        subject_column_ref = ""
        for i in range(len(subject_column)):
            subject_column_ref += "$({})".format(col_names[subject_column[i]])
            if i < len(subject_column) - 1:
                subject_column_ref += "/"

    # If Subject Column is of IRI type, we can use a reference to the IRIs, 
    # since subject columns are always Named Entity Columns
    primary_condition = isinstance(subject_column, int)
    secondary_condition = secondary_annotations[subject_column] == "URL"
    if primary_condition and secondary_condition and cea == []:
        subjectTM["s"] = subject_column_ref
        return subjectTM

    # If CEA are available, a reference to the annotation IRIs is used
    if cea:
        subjectTM["s"] = subject_column_ref
        return subjectTM

    # If CEA are not available, a dummy template value is used 
    # This can then be manually processed, be replaced with a blank node 
    # or a mapping function that calls a service for retrieving CEA
    else:
        entity_template_prefix = "http://example.com/entity/"
        subjectTM["s"] = entity_template_prefix + subject_column_ref
        return subjectTM


# Definition of the Subject Map for the Named Entity Object Columns of the
# table
def define_object_column_subject_map(objectTM, object_column,
                                     primary_annotations,
                                     secondary_annotations, cea, col_name):

    # If Object Column is a Named Entity Column and is of IRI type,
    # we can use a reference to the IRIs
    primary_condition = primary_annotations[object_column] == "NE"
    secondary_condition = secondary_annotations[object_column] == "URL"
    if primary_condition and secondary_condition:
        objectTM["s"] = "$({})".format(col_name)
        return objectTM

    # If CEA are available, a reference to the annotation IRIs is used
    if cea:
        objectTM["s"] = "$({})".format(col_name)
        return objectTM

    # If CEA are not available, a dummy template value is used 
    # This can be manually processed by the user or be replaced with 
    # a mapping function that calls a service for retrieving CEA
    else:
        entity_template_prefix = "http://example.com/entity/"
        objectTM["s"] = entity_template_prefix + "$({})".format(col_name)
        return objectTM



import re


# Define the Predicate-Object Maps for the Subject Column
def define_subject_column_po_map(subjectTM, subject_column,
                                 primary_annotations, secondary_annotations,
                                 cea, cpa, cta, col_names):

    property_template_prefix = "http://example.com/property/"
    type_template_prefix = "http://example.com/entityType/"
    entity_template_prefix = "http://example.com/entity/"
    rdf_type_shortcut = "a"
    cta_label = None

    subjectTM["po"] = []

    # Definition of rdf:type Predicate-Object Map
    if subject_column == -1 or subject_column is None or not cta:
        # If the Subject Column does not exist or if we do not have
        # CTA information, we use a dummy template rdf type.
        # This can then be manually processed, be replaced with a blank node
        # or a mapping function that calls a service for retrieving CTA
        subjectTM["po"].append({"p": rdf_type_shortcut,
                                "o": type_template_prefix + "subjectType"})
    else:
        for cta_data in cta:
            first_pred = int(cta_data[0]) == subject_column
            second_pred = [int(cta_data[0])] == subject_column
            if first_pred or second_pred:
                cta_label = cta_data[1]
                break
        if cta_label:
            subjectTM["po"].append({"p": rdf_type_shortcut,
                                    "o": cta_label})
        else:
            subjectTM["po"].append({"p": rdf_type_shortcut,
                                    "o": type_template_prefix + "subjectType"})

    # Definition of the rest of Predicate-Object Maps for table without a
    # subject column
    # All columns can potentially be Object Columns. We decide according to
    # available information
    # CPA is not possible without a subject column so we use dummy constant
    # properties to define the predicate maps.
    # These can then be manually processed or retrieved by a mapping function
    if subject_column == -1 or subject_column is None:

        for i in range(len(primary_annotations)):

            # If a Named Entity Object Column is of type IRI, we can use its
            # IRIs as reference to define the object maps
            primary_if = primary_annotations[i] == "NE"
            secondary_if = secondary_annotations[i] == "URL"
            if primary_if and secondary_if and cea == []:
                subjectTM["po"].append({"p": property_template_prefix +
                                        "s-{}".format(
                                            ensure_valid_url(col_names[i])),
                                        "o": "$({})~iri".format(col_names[i])})
                continue

            # Named Entity Object Columns
            if primary_annotations[i] == "NE":
                # If CEA are available, a reference to the annotation IRIs is
                # used
                if cea:
                    subjectTM["po"].append({"p": property_template_prefix +
                                            "s-{}".format(
                                                ensure_valid_url(
                                                    col_names[i])),
                                            "o": [{"value": "$({})".format(
                                                ensure_valid_url(
                                                    col_names[i])),
                                                   "type": "iri"}]})

                # If CEA are not available, a dummy template value is used
                # This can then be manually processed, be replaced with a blank
                # node or a mapping function that calls a service for
                # retrieving CEA
                else:
                    value = entity_template_prefix + \
                            "$({})".format(ensure_valid_url(col_names[i]))
                    subjectTM["po"].append({"p": property_template_prefix +
                                            "s-{}".format(ensure_valid_url
                                                          (col_names[i])),
                                            "o": [{"value": value,
                                                   "type": "iri"}]})

            # Literal Object Columns
            elif primary_annotations[i] == "L":

                # For literal columns we can add a reference to the column.
                # This can be manually replaced with a template, according to
                # the use case.
                # If datatypes are also available we can add them as well.
                xsd_type = map_datatype_to_xsd(secondary_annotations[i])
                if xsd_type is None:
                    subjectTM["po"].append({"p": property_template_prefix +
                                            "s-{}".format(ensure_valid_url
                                                          (col_names[i])),
                                            "o": [{"value":  "$({})"
                                                   .format(col_names[i])}]})
                else:
                    subjectTM["po"].append({"p": property_template_prefix +
                                            "s-{}".format(ensure_valid_url
                                                          (col_names[i])),
                                            "o": [{"value":  "$({})"
                                                   .format(col_names[i]),
                                                   "datatype": xsd_type}]})

    # Else if the table has a subject column
    else:
        if isinstance(subject_column, int):
            subject_column = [subject_column]

        # If we have CPA annotations use them to define the predicate map
        if cpa:

            for s, o, p in cpa:

                # Check for missing annotations
                if p is None or p[-3:] == "/P1":
                    p = property_template_prefix + \
                        "{}-{}".format(ensure_valid_url(
                            col_names[int(s)]),
                                       ensure_valid_url(col_names[int(o)]))

                if int(s) in subject_column:

                    # If a Named Entity Object Column is of type IRI, we can
                    # use its IRIs as reference to define the object maps
                    primary_if = primary_annotations[int(o)] == "NE"
                    secondary_if = secondary_annotations[int(o)] == "URL"
                    if primary_if and secondary_if and cea == []:
                        subjectTM["po"].append({"p": p,
                                                "o": "$({})~iri"
                                                .format(col_names[int(o)])})
                        continue

                    # If the Object Column is a Named Entity Column
                    if primary_annotations[int(o)] == "NE":

                        # If CEA are available, a reference to the annotation
                        # IRIs is used
                        if cea:
                            subjectTM["po"].append({"p": p,
                                                    "o": [{"value": "$({})"
                                                           .format(col_names
                                                                   [int(o)]),
                                                           "type": "iri"}]})

                        # If CEA are not available, a dummy template value is
                        # used
                        # This can then be manually processed, be replaced
                        # with a blank node or a mapping function that calls a
                        # service for retrieving CEA
                        else:
                            value = entity_template_prefix + \
                                    "$({})".format(col_names[int(o)])
                            subjectTM["po"].append({"p": p,
                                                    "o": [{"value": value,
                                                           "type": "iri"}]})

                    # If the Object Column is a Literal Column
                    elif primary_annotations[int(o)] == "L":

                        # For literal columns we can add a reference to the
                        # column.
                        # This can be manually replaced with a template,
                        # according to the use case.
                        # If datatypes are also available we can add them as
                        # well.
                        xsd_type = map_datatype_to_xsd(
                            secondary_annotations[int(o)])
                        if xsd_type is None:
                            subjectTM["po"].append({"p": p,
                                                    "o": [{"value":  "$({})"
                                                           .format(col_names
                                                                   [int(o)]
                                                                   )}
                                                          ]})
                        else:
                            subjectTM["po"].append({"p": p,
                                                    "o": [{"value":  "$({})"
                                                           .format(col_names
                                                                   [int(o)]),
                                                           "datatype":
                                                               xsd_type}]})

        # If we do not have CPA annotations we use dummy constant properties
        # to define the predicate maps. These can then be manually processed or
        # retrieved by a mapping function
        elif not cpa:

            for i in range(len(primary_annotations)):

                if i in subject_column:
                    continue

                # If a Named Entity Object Column is of type IRI, we can use
                # its IRIs as reference to define the object maps
                primary_if = primary_annotations[i] == "NE"
                secondary_if = secondary_annotations[i] == "URL"
                if primary_if and secondary_if and cea == []:
                    subjectTM["po"].append({"p": property_template_prefix +
                                            "s-{}".format(ensure_valid_url(
                                                col_names[i])),
                                            "o": "$({})~iri"
                                            .format(col_names[i])})
                    continue

                # Named Entity Object Columns
                if primary_annotations[i] == "NE":
                    # If CEA are available, a reference to the annotation IRIs
                    # is used
                    if cea:
                        subjectTM["po"].append({"p": property_template_prefix +
                                                "s-{}".format(ensure_valid_url
                                                              (col_names[i])),
                                                "o": [{"value": "$({})"
                                                       .format(col_names[i]),
                                                       "type": "iri"}]})

                    # If CEA are not available, a dummy template value is used
                    # This can then be manually processed, be replaced with a
                    # blank node or a mapping function that calls a service for
                    # retrieving CEA
                    else:
                        value = entity_template_prefix + \
                                "$({})".format(ensure_valid_url(col_names[i]))
                        subjectTM["po"].append({"p": property_template_prefix +
                                                "s-{}".format(ensure_valid_url
                                                              (col_names[i])),
                                                "o": [{"value": value,
                                                       "type": "iri"}]})

                # Literal Object Columns
                elif primary_annotations[i] == "L":

                    # For literal columns we can add a reference to the column.
                    # This can be manually replaced with a template, according
                    # to the use case. If datatypes are also available we can
                    # add them as well.
                    xsd_type = map_datatype_to_xsd(secondary_annotations[i])
                    if xsd_type is None:
                        value = "$({})".format(col_names[i])
                        subjectTM["po"].append({"p": property_template_prefix +
                                                "s-{}".format(
                                                    ensure_valid_url(
                                                        col_names[i])),
                                                "o": [{"value":  value}]})
                    else:
                        subjectTM["po"].append({"p": property_template_prefix +
                                                "s-{}".format(
                                                    ensure_valid_url(
                                                        col_names[i])),
                                                "o": [{"value":  "$({})"
                                                       .format(col_names[i]),
                                                       "datatype": xsd_type}]})

    return subjectTM


# Define the Predicate-Object Maps for the Subject Column
def define_object_column_po_map(objectTM, column, cta, col_name):

    type_template_prefix = "http://example.com/entityType/"
    rdf_type_shortcut = "a"
    cta_label = None

    objectTM["po"] = []

    # Definition of rdf:type Predicate-Object Map
    if not cta:
        # If do not have CTA information, we use a dummy template rdf type.
        # This can then be manually processed, be replaced with a blank node
        # or a mapping function that calls a service for retrieving CTA
        objectTM["po"].append({"p": rdf_type_shortcut,
                               "o": type_template_prefix +
                               ensure_valid_url(col_name)})
    else:
        for cta_data in cta:
            if int(cta_data[0]) == column:
                cta_label = cta_data[1]
                break
        if cta_label:
            objectTM["po"].append({"p": rdf_type_shortcut,
                                   "o": cta_label})
        else:
            objectTM["po"].append({"p": rdf_type_shortcut,
                                   "o": type_template_prefix +
                                   ensure_valid_url(col_name)})

    return objectTM


def ensure_valid_url(string):

    # Replace spaces and remove any non-alphanumeric characters except
    # underscores
    string = re.sub(r'\s+', '_', string)
    string = re.sub(r'[^a-zA-Z0-9_]', '', string)

    return string


def map_datatype_to_xsd(secondary_annotation):

    default_datatype = None

    # Mapping of common secondary annotation to xsd datatypes
    xsd_mappings = {
        "BOOLEAN": "xsd:boolean",
        "INT": "xsd:int",
        "FLOAT": "xsd:float",
        "STRING": "xsd:string",
        "TIME": "xsd:string",
        "PHONE": "xsd:string",
        "URL": "xsd:anyURI",
        "EMAIL": "xsd:string",
        "IP": "xsd:string",
        "HEX": "xsd:string",
        "CREDIT_CARD": "xsd:string",
        "ADDRESS": "xsd:string",
        "COORDS": "xsd:string",
        "ISBN": "xsd:string",
    }

    if secondary_annotation in xsd_mappings:
        return xsd_mappings[secondary_annotation]
    else:
        return default_datatype



import requests
import pprint
import time
import sys
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


def mtab(file):

    print("Querying the MTab API to retrieve semantic table annotations...")
    api_url = "https://mtab.kgraph.jp/api/v1/mtab"

    retry_count = 10
    backoff_factor = 2
    session = requests.Session()

    # Open the file in binary mode
    with open(file, 'rb') as table:
        # Create a dictionary for the file parameter to send
        files = {'file': table}

        # Make the POST request to the API
        response = None
        retrieve_success = False
        for attempt in range(retry_count):
            try:
                response = session.post(api_url, files=files, verify=False,
                                        timeout=(5, 10000))
                response.raise_for_status()
                retrieve_success = True
                break
            except requests.exceptions.RequestException as e:
                print(f"Attempt {attempt + 1} failed: {e}")
                wait_time = backoff_factor**attempt + (time.perf_counter() % 1)
                time.sleep(wait_time)
                retrieve_success = False

    # Check the status of the request
    if retrieve_success is True:
        # Parse the JSON response (assuming the API returns JSON)
        annotations = response.json()

        if annotations["tables"][0]["status"] == "Error":
            (subject_column, primary_annotations, secondary_annotations,
             new_cea, new_cpa, new_cta, cqa) = mtab(file)
        else:
            (subject_column, primary_annotations, secondary_annotations,
             new_cea, new_cpa, new_cta, cqa) =\
                 standard_annotation_formatter(annotations)
    else:
        sys.exit("Failed to annotate. Status code: "
                 f"{response.status_code}, Response: {response.text}")

    return (subject_column, primary_annotations, secondary_annotations,
            new_cea, new_cpa, new_cta, cqa)


def standard_annotation_formatter(annotations):

    # pprint.pprint(annotations)

    # Get subject column 
    subject_column =\
        int(annotations["tables"][0]["structure"]["core_attribute"])

    # Get CEA labels
    cea = annotations["tables"][0]["semantic"]["cea"]
    new_cea = []
    for cea_annotation in cea:
        if cea_annotation["target"][0] == 0:
            continue
        new_cea.append([str(cea_annotation["target"][0]),
                        str((cea_annotation["target"][1])),
                        cea_annotation["annotation"]["wikidata"]])

    # Get CPA labels
    cpa = annotations["tables"][0]["semantic"]["cpa"]
    new_cpa = []
    for cpa_annotation in cpa:
        new_cpa.append([str(cpa_annotation["target"][0]),
                        str(cpa_annotation["target"][1]),
                        cpa_annotation["annotation"][0]["wikidata"]])

    # Get CTA labels
    cta = annotations["tables"][0]["semantic"]["cta"]
    new_cta = []
    ne_columns = []
    for cta_annotation in cta:
        ne_columns.append(int(cta_annotation["target"]))
        new_cta.append([str(cta_annotation["target"]),
                        cta_annotation["annotation"][0]["wikidata"]])

    # Get primary and secondary annotations
    n_cols = int(annotations["tables"][0]["structure"]["columns"])
    primary_annotations = ["L"]*n_cols
    secondary_annotations = ["Unknown"]*n_cols
    for i in range(len(primary_annotations)):
        if i in ne_columns: 
            primary_annotations[i] = "NE"
            secondary_annotations[i] = "NE"

    new_cpa = fill_missing_properties(new_cpa, subject_column,
                                      primary_annotations)
    new_cta = fill_missing_types(new_cta, subject_column, primary_annotations)
    cqa = []

    return (subject_column, primary_annotations, secondary_annotations,
            new_cea, new_cpa, new_cta, cqa)


def fill_missing_properties(cpa, subject_column, primary_annotations):

    object_columns = []
    for i in range(len(primary_annotations)):
        if i != subject_column:
            object_columns.append(i)

    object_columns_with_properies = []
    for property_annotation in cpa:
        object_columns_with_properies.append(int(property_annotation[1]))

    object_columns_without_properties = \
        [item for item in object_columns
         if item not in object_columns_with_properies]
    for object_column in object_columns_without_properties:
        cpa.append([str(subject_column), str(object_column), None])

    return cpa


def fill_missing_types(cta, subject_column, primary_annotations):

    ne_cols = []
    for i in range(len(primary_annotations)):
        if primary_annotations[i] == "NE" or i == subject_column:
            ne_cols.append(i)

    ne_cols_with_type = []
    for type_annotation in cta:
        ne_cols_with_type.append(int(type_annotation[0]))

    ne_cols_without_type =\
        [item for item in ne_cols if item not in ne_cols_with_type]
    for ne_column in ne_cols_without_type:
        cta.append([str(ne_column), None])

    return cta



from autorml.annotation.torchic_tab import torchic_tab
from autorml.annotation.mtab import mtab
from autorml.mapping_synthesis import mapping_synthesis, rml_generation
from autorml.materialize import rdf_generation
from autorml.logical_source import delete_semantic_table
from autorml.annotation.save_annotations import save_annotations

import sys
import argparse
import os


def define_args():

    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--input_table",
                        required=True,
                        type=str,
                        help="Input table in CSV format")

    parser.add_argument("-oy", "--yarrml_output",
                        default="mappings.yml",
                        type=str,
                        help="Output YARRRML mappings")

    parser.add_argument("-or", "--rml_output",
                        default="mappings.rml.ttl",
                        type=str,
                        help="Output RML mappings")

    parser.add_argument("-okg", "--rdf_output",
                        default="kg.nt",
                        type=str,
                        help="Output RDF file")

    parser.add_argument("-mf", "--mappings_folder",
                        default="mappings",
                        type=str,
                        help="Output mappings folder")

    parser.add_argument("-m", "--materialize",
                        action='store_true',
                        help="Generate RDF")

    parser.add_argument("-rf", "--rdf_folder",
                        default="rdf",
                        type=str,
                        help="Output RDF folder")

    parser.add_argument("-sta", "--sta_system",
                        default="torchictab",
                        type=str,
                        help="Used semantic table annotation system")

    parser.add_argument("-sa", "--save_annotations",
                        action='store_true',
                        help="Save semantic table annotation system results")

    parser.add_argument("-af", "--annotations_folder",
                        default="annotations",
                        type=str,
                        help="Output annotations folder")

    parser.add_argument("-oa", "--annotations_output",
                        default="annotations.json",
                        type=str,
                        help="Output annotations mappings")

    parser.add_argument("-ds", "--delete_sem",
                        action='store_true',
                        help="Delete supporting semantically enhanced"
                        " table after termination")

    return parser


def main():

    print("AutoRML initialized!\n")
    args = define_args().parse_args()

    # Extract annotations from Semantic Annotation System
    if args.sta_system == "mtab":
        print("Annotating table using MTab:", args.input_table, "...")
        (subject_column, primary_annotations, secondary_annotations,
         cea, cpa, cta, cqa) = mtab(args.input_table)

    elif args.sta_system == "torchictab":
        print("Annotating table using TorchicTab:", args.input_table, "...")
        (subject_column, primary_annotations, secondary_annotations,
         cea, cpa, cta, cqa) = torchic_tab(args.input_table)
    else:
        sys.exit("Selected annotation system not supported. Exiting...")
    print("Tabular annotation completed!\n")

    if (args.save_annotations):
        if not os.path.exists(args.annotations_folder):
            os.makedirs(args.annotations_folder)
            print(f"Folder '{args.annotations_folder}' created.")

        save_annotations(subject_column, primary_annotations,
                         secondary_annotations, cea, cpa, cta, cqa,
                         os.path.join(args.annotations_folder,
                                      args.annotations_output))

    if not os.path.exists(args.mappings_folder):
        os.makedirs(args.mappings_folder)
        print(f"Folder '{args.mappings_folder}' created.")

    mapping_synthesis(args.input_table,
                      os.path.join(args.mappings_folder, args.yarrml_output),
                      subject_column, primary_annotations,
                      secondary_annotations, cea, cpa, cta, cqa)

    rml_generation(os.path.join(args.mappings_folder, args.yarrml_output),
                   os.path.join(args.mappings_folder, args.rml_output))
    print("\nYARRRML and RML mappings succesfully generated!")

    if (args.materialize):

        if not os.path.exists(args.rdf_folder):
            os.makedirs(args.rdf_folder)
            print(f"Folder '{args.rdf_folder}' created.")

        rdf_generation(os.path.join(args.rdf_folder, 'config.ini'),
                       os.path.abspath(
            str(os.path.join(args.mappings_folder, args.rml_output))),
                       args.rdf_output)
        print("\nRDF graph succesfully generated!")

    if (args.delete_sem):
        # Delete semantic table created for including semantic cell labels
        delete_semantic_table(os.path.abspath(
            args.input_table.replace(".csv", "-semantic.csv")))

    return


if __name__ == "__main__":

    main()



from autorml.logical_source import define_source
from autorml.term_maps import define_term_maps

from ruamel.yaml import YAML
import yatter


def mapping_synthesis(table, yarrrml_output_location,
                      subject_column, primary_annotations,
                      secondary_annotations, cea, cpa, cta, cqa):

    # Initialize YAML file
    yaml = YAML(pure=True)
    yaml.default_flow_style = False
    yaml.indent(mapping=2, sequence=4, offset=2)

    mappings = {"authors": "AutoRML"}
    mappings, col_names = define_source(mappings, table, cea,
                                        primary_annotations)
    mappings = define_term_maps(mappings,
                                subject_column, primary_annotations,
                                secondary_annotations,
                                cea, cpa, cta,
                                col_names)
    # mappings = define_subject_maps(mappings, subject_column, cea)

    # Create the YARRRML mapping file with the dictionary assembled
    with open(yarrrml_output_location, 'w', encoding="utf-8") as file:
        for section in mappings:
            yaml.dump({section: mappings[section]}, file, )
            file.write('\n')

    print("YARRRML mappings succesfully created!")

    return


def rml_generation(yarrrml_output_location, rml_output_location):

    yaml = YAML(typ='safe', pure=True)
    rml_content = yatter.translate(yaml.load(open(yarrrml_output_location)))

    output_file = open(rml_output_location, "w")
    output_file.write(rml_content)
    output_file.close()
    print("RML mappings succesfully created!")

    return



from torchic_tab_heuristic.task_calculation_wd import generate_annotations #Closed source. Contact for access


#Call of TorchicTab to extract tabular annotations for a given table
def torchic_tab(file):

    es_index = False
    (subject_column, primary_annotations, secondary_annotations,
     cea, cpa, cta) = generate_annotations(file, es_index)
    cqa = []

    return (subject_column, primary_annotations, secondary_annotations, 
            cea, cpa, cta, cqa)



import json


def save_annotations(subject_column, primary_annotations,
                     secondary_annotations, cea, cpa, cta, cqa, path):

    annotation_object = {
            "subject_column": subject_column,
            "primary_annotations": primary_annotations,
            "secondary_annotations": secondary_annotations,
            "cea": cea,
            "cpa": cpa,
            "cta": cta,
            "cqa": cqa
        }

    with open(path, 'w') as file:
        json.dump(annotation_object, file, indent=4)



from setuptools import setup

with open("requirements.txt") as f:
    required = f.read().splitlines()

setup(
    name='autorml',
    version='0.1.3',
    packages=["autorml", "autorml.annotation"],
    install_requires=required,
    author="Ioannis Dasoulas",
    author_email="ioannis.dasoulas@kuleuven.be",
    description="AutoRML: A framework for automatic RML mapping generation "
                "using semantic table annotations",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/dtai-kg/AutoRML",
    license="Apache-2.0",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
    ],
    python_requires='>=3.9, <3.12'
)



import morph_kgc
import configparser
import os


def rdf_generation(config_path, rml_path, kg_name):

    create_ini_file(config_path, rml_path, kg_name)
    rdf_graph = morph_kgc.materialize(config_path)
    rdf_graph.serialize(destination=os.path.join
                        (os.path.abspath
                         (os.path.dirname(config_path)), kg_name),
                        format="nt", encoding="utf-8")

    return


def create_ini_file(config_path, rml_path, kg_name):
    # Create a ConfigParser object
    config = configparser.ConfigParser()

    # Add a section and set a key-value pair
    config['CONFIGURATION'] = {'logging_level': 'INFO',
                               'output_file': os.path.join(
                                   os.path.abspath(
                                       os.path.dirname(config_path)), kg_name),
                               'output_format': 'N-TRIPLES'}
    config['DataSource1'] = {'mappings': rml_path}

    # Write the configuration to a .ini file
    with open(config_path, 'w') as configfile:
        config.write(configfile)
